--[[

This file is part of RampageAI.

RampageAI is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

RampageAI is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with RampageAI; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

--]]


--------------------------------------
-- Initialize Auto-Alchemist Skills --
--------------------------------------

function InitOwnerSkills()
	-- Check if the user wants to use auto Homunculus Resurrection
	if AutoRes then
		-- Set the HomuResuCheck function
		HomuResuCheck = AI_HomuResurrect

		-- Add a function that calls HomuResuCheck (so HomuResuCheck can be hooked)
		AddAlchSkill(10,function(castAllowed)
			-- Call HomuResuCheck
			return HomuResuCheck(castAllowed)
		end)
	end

	-- Check if the user wants to cheat. I mean auto-aid-potion. (I'm kidding!)
	if AutoAidPot then
		-- Check if we're using the advanced aid potion code
		if AidPotAdvanced then
			-- Setup a structure to keep track of which potions caused
			-- skill failure
			local AidPotTriesResetFunction = function()
								AidPotTriesLeft = { AidPotTriesPerLevel,
								AidPotTriesPerLevel, AidPotTriesPerLevel,
								AidPotTriesPerLevel }
			end
			AidPotTriesResetFunction()
	
			-- There is no current try
			AidPotAttempted = 0
	
			-- Setup a timeout to reset the tries
			if AidPotTriesReset > 0 then
				Timeout.add(AidPotTriesReset, AidPotTriesResetFunction, true)
			end
	
			-- Make sure we won't get any divide by 0 errors
			if AidPotIncrement == 0 then
				-- They asked for it! Its what every other 0
				-- amount turns into!
				AidPotIncrement = 100
			end
	
			-- Check if highest is lower than lowest
			if AidPotHighest < AidPotLowest then
				-- Swap 'em
				local temp = AidPotHighest
				AidPotHighest = AidPotLowest
				AidPotLowest = temp
			end
	
			-- Set the timeout for AidPot
			AidPotTimeout = 0
	
			-- Set the AidPotCheck function
			AidPotCheck = AI_AidPotion_Adv
		else
			-- Set the AidPotCheck function
			AidPotCheck = AI_AidPotion_Basic
		end

		-- Add a function to our alchemist skills table that calls AidPotCheck
		AddAlchSkill(20,function(castAllowed)
			-- Call AidPotCheck
			return AidPotCheck(castAllowed)
		end)
	end

	-- Check if they want to support heal anybody
	if AutoHealFriends > 0 or AutoHealTempFriends > 0 or
		AutoHealAnybody > 0 then
		-- Add a function to check if a previous heal was successful
		AddAlchSkill(22,AI_AutoSupportHealCheck)
	end

	-- Check if they want to heal friends
	if AutoHealFriends > 0 then
		-- Add a function to heal perm friends
		AddAlchSkill(23,function(castAllowed)
			-- Call AI_AutoSupportHeal
			return AI_AutoSupportHeal(castAllowed,PermFriendsOnScr,"friend",AutoHealFriends)
		end)
	end

	-- Check if they want to heal temporary friends
	if AutoHealTempFriends > 0 then
		-- Add a function to heal temp friends
		AddAlchSkill(23,function(castAllowed)
			-- Call AI_AutoSupportHeal
			return AI_AutoSupportHeal(castAllowed,TempFriends,"temp friend",AutoHealTempFriends)
		end)
	end

	-- Check if they want to heal anybody in range
	if AutoHealAnybody > 0 then
		-- Add a function to heal anybody
		AddAlchSkill(23,function(castAllowed)
			-- Call AI_AutoSupportHeal
			return AI_AutoSupportHeal(castAllowed,ActorData,"player",AutoHealAnybody)
		end)
	end

	-- Check if they want to use blue potions
	if AutoAidBlue then
		-- Set the function
		AidPotBlueCheck = AI_AidPotion_Blue

		-- Add a function to our alchemist skills table
		AddAlchSkill(25,function(castAllowed)
			-- Call AidPotBlueCheck
			return AidPotBlueCheck(castAllowed)
		end)
	end

	-- Check if they want us to try to move closer for potions
	if AidPotMoveCloser then
		AidPot_MoveCloser = AI_AidPot_MoveCloser
	end

	-- Default to not acquiring targets for the alchemist
	local alchTarg = false

	-- Check if they want to throw acid bombs
	if AutoAcidDemo then
		AddAlchSkill(27,AI_AutoAcidBomb)
		alchTarg = true
	end

	-- Check if they want to throw acid terrors
	if AutoAcidTerror then
		AddAlchSkill(28,AI_AutoAcidTerror)
		alchTarg = true
	end

	-- Check if they want to auto-cart revolution
	if AutoSplashCart then
		-- Add the auto-cart-revo function
		AddAlchSkill(30,AI_CartRevo)
		alchTarg = true
	end

	-- Check if they want to auto-magnum break
	if AutoSplashBlast then
		-- Add the auto-magnum break function
		AddAlchSkill(30,AI_MagnumBreak)
		alchTarg = true
	end

	-- Check if they want to automatically mammonite
	if AutoMammonite then
		AddAlchSkill(30,AI_AutoMammo)
		alchTarg = true
	end

	-- Check if they want to use bolts automatically
	if AutoBoltSword then
		-- Set the AutoBolt function (so custom scripts can hook it)
		AutoBoltCheck = AI_AutoBolt

		-- Add the auto-bolt function
		AddAlchSkill(35,function(castAllowed)
			-- Call AutoBoltCheck (to allow script to hook)
			return AutoBoltCheck(castAllowed)
		end)

		alchTarg = true
	end

	-- Check if they want to use bash automatically
	if AutoCutlusBash then
		AddAlchSkill(35,AI_AutoBash)
		alchTarg = true
	end

	-- Check if they want to throw a tomahawk
	if AutoTomahawk then
		AddAlchSkill(35,AI_ThrowTomahawk)
		alchTarg = true
	end

	-- Check if they want to heal bomb automatically
	if AutoHealBomb then
		-- Add the auto-heal function
		AddAlchSkill(35,AI_AutoHealBomb)
		alchTarg = true
	end

	-- Check if we should be targetting for the alchemist
	if alchTarg then
		-- We are set to use some offensive skills
		AlchAcquireTarget = AI_AlchAcquireTarget
	else
		-- We don't want to waste time selecting a target
		AlchAcquireTarget = NegativeReturn
	end

	-- Calculate the casting-time modifier
	CastModifier = 1.0 - (AlchemistDex/150)

	-- Check if they're wearing a phen
	if AlchemistPhen then
		-- Multiply the cast modifier by 125%
		CastModifier = CastModifier * 1.25

		-- Use the phen version of owner motion
		GetOwnerMotion = AI_GetOwnerMotion_Phen
	else
		-- Use the non-phen owner motion
		GetOwnerMotion = AI_GetOwnerMotion_NoPhen
	end

	-- Set the check alchemist skills function
	CheckAlchSkills = AI_CheckAlchSkills
end

-- Function to add skills to the alch skills priority table
function AddAlchSkill(priority,func)
	-- Check that func is a function
	if type(func) ~= "function" then
		return
	end

	-- Create the object we're adding
	local obj = { }
	obj.priority = priority
	obj.func = func

	-- Loop through our table finding a priority
	local i
	for i=Table.getn(AlchSkillsTable),1,-1 do
		-- Check the priority of the current function
		if priority >= AlchSkillsTable[i].priority then
			-- Insert our object
			Table.insert(AlchSkillsTable,i+1,obj)

			-- Don't loop anymore
			return
		end
	end

	-- Since we didn't find a place for it, add it at the front
	Table.insert(AlchSkillsTable,1,obj)
end

-------------------------------
-- Auto-Use Alchemist Skills --
-------------------------------

function AI_CheckAlchSkills(castAllowed)
	-- Check castAllowed
	if castAllowed == nil then
		castAllowed = true
	elseif not castAllowed then
		castAllowed = false
	end

	-- Loop through our table of alchemist skills
	--	** Idea for priority table from Katrina of RagnaInfo forums **
	local i
	for i=1,Table.getn(AlchSkillsTable) do
		-- Call the function and update the castAllowed
		castAllowed = AlchSkillsTable[i].func(castAllowed)
	end

	return castAllowed
end

----------------------------------
-- Auto-Homunculus Resurrection --
----------------------------------

-- (Idea by Spike Sydney)

-- Function to check (and use) Homunculus Resurrection if necessary
function AI_HomuResurrect(castAllowed)

	--[[
		Hidden option

		This specifies how long after it tries to Auto-Resurrect that
		it'll allow the Homunculus to move outside of the 2 tile range
	--]]
	local MaxDistanceDelay = 1000

	-- Get the ticks
	local ticks = GetTick()

	-- Check if its time to reset MaxDistance
	if MaxDistance ~= 0 and MaxDistanceTimeout < ticks then
		MaxDistance = 0
	end

	-- Check if our owner is sitting down.
	if OwnerMotion == MOTION_SIT then
		-- We can't do anything :(
		return castAllowed
	end

	-- Check if the homunculus is losing life
	if HomunHPDelta.iteration >= 0 and HomunHPDelta.second >= 0 then
		-- Not losing life, so why bother?
		return castAllowed
	end

	--[[ SPUsable automatically deducts 50sp from the Owner's SP pool
		so that other skills don't use it. If we do in fact have at
		least 50 SP, then we will break even at 0 or above
	--]]
	if SPUsable(OwnerID) < 0 then
		-- We don't have enough SP, allow aid pot
		return castAllowed
	end

	-- Check if the Homunculus's HP is below a certain amount
	if ActorHP(HomunID) >= AutoResHP then
		-- Check if we need to "intelligently" cast Homunculus Resurrection
		if AutoResSmart then
			-- Grab the amount of damage that the homunculus is taking per second/iteration
			local sec = HomunHPDelta.second
			local iter = HomunHPDelta.iteration
			local hp = GetV(V_HP,HomunID)

			-- Nullify any positives
			if iter > 0 then
				iter = 0
			end
			if sec > 0 then
				sec = 0
			end

			-- Make the absolute values
			sec = math.abs(sec)
			iter = math.abs(iter)

			-- Check the distance of the homunculus
			local dist = AADistance(HomunID,OwnerID) + 2

			-- Multiply the damage based on distance
			sec = sec * math.ceil(dist / 2)
			iter = iter * dist
			
			-- Check if the homunculus won't die within the time it'll get close enough
			-- to auto-resurrect itself
			if hp > iter and hp > sec then
				-- We probably won't die, see if we should aid pot
				return castAllowed
			end

			if hp < sec then
				Log(4,"Smart routine flagged auto-res (second delta)")
			else
				Log(4,"Smart routine flagged auto-res (iteration delta)")
			end
		else
			-- We aren't casting resurrection, so continue the allowed
			return castAllowed
		end

	else
		Log(4,"Dumb routine flagged auto-res")
	end

	-- Check if the Homunculus is in range of resurrection (range 2, we move to range 1)
	local dist = AADistance(HomunID,OwnerID)
	if dist > 1 then
		-- Check if we're not already moving towards our owner
		local seq = { SEQ_NONE }
		if List.size(AI_seq) > 0 then
			seq = AI_seq[AI_seq.first]
		end
		if seq[1] ~= SEQ_MOVE_TO_POS or seq[2] ~= OwnerID or seq[3] ~= 1 then
			-- Log it
			Log(1,"Moving within 1 tile of Owner, so we can Auto-Homunculus Resurrection")
	
			-- Create a sequence
			local seq2 = { SEQ_MOVE_TO_POS, OwnerID, 1, loglevel=6 }
	
			-- Begin the command
			local dummy
			dummy,seq2 = SEQ_AMove_Pos(seq2)
	
			-- Add it to the front of the sequence queue
			if seq2 ~= nil then
				List.pushleft(AI_seq,seq2)
			end
		end

		-- Check the distance of the homunculus
		local dist = AADistance(HomunID,OwnerID)

		-- Set MaxDistance (and it'll wear off after a delay)
		MaxDistance = 1
		MaxDistanceTimeout = ticks + math.floor(MaxDistanceDelay * dist / 3)

		-- While the homunculus is out of range of resurrection, allow aid pot
		if dist > 2 then
			return castAllowed
		end
	end

	-- If the cast is not allowed, we're still moving closer for when it is allowed
	if not castAllowed then
		-- ... but we're going to pass on that casting isn't allowed

		-- Log it
		Log(4,"Not allowed to cast Homunculus Resurrection yet, still moving closer.")

		-- Return false
		return false
	end
	
	-- Make sure we're ready to cast
	if OwnerMotion ~= MOTION_CASTING and OwnerMotion ~= MOTION_SKILL then

		-- Log it
		Log(1,"Attempting to auto-cast Homunculus Resurrection")

		-- Use the skill
		SkillObject(HomunID, 5, 247, HomunID)

		-- Set our cast time and distance timeout
		OwnerSkillTimeout = ticks + (CastModifier * 3000) + AutoResDelay
		MaxDistanceTimeout = OwnerSkillTimeout + MaxDistanceDelay

	else
		-- Log it
		--Log(0,"Not casting Homunculus Resurrect because we're not ready yet")

		-- Set our distance timeout
		MaxDistanceTimeout = ticks + MaxDistanceDelay
	end

	-- Move the Homunculus to itself to prevent tile-lagging
	BypassMove(HomunID,GetV(V_POSITION,HomunID))

	-- Make sure they stay within 1 tile
	MaxDistance = 1

	-- We're using Resurrection, so don't try to Aid Pot
	return false
end


---------------------
-- Auto-Aid-Potion --
---------------------

-- Function to move closer to the alchemist if the homun needs to be potted
function AI_AidPot_MoveCloser()
	-- Get the next sequence
	local seq
	if List.size(AI_seq) > 0 then
		seq = AI_seq[AI_seq.first]
	else
		seq = { SEQ_NONE }
	end

	-- Check if its a move to our owner
	if (seq[1] == SEQ_MOVE_TO or seq[1] == SEQ_MOVE_TO_POS or
		seq[1] == SEQ_MOVE_TO_NEG) and seq[2] == OwnerID and
		math.abs(seq[3]) <= 6
	then
		-- Its a move to our owner, just wait for it
		return castAllowed
	end

	-- Add a movement to our owner
	List.pushleft(AI_seq,{ SEQ_MOVE_TO, OwnerID, 5 } )
end

-- Function to check (and use) Aid Potion if necessary
function AI_AidPotion_Basic(castAllowed,blue)
	-- Check if a cast is allowed
	if not castAllowed then
		return castAllowed
	end

	-- Check to see if our owner is sitting down
	if OwnerMotion == MOTION_SIT then
		return castAllowed
	end

	-- Get the tick count
	local ticks = GetTick()

	-- Check if our owner walked recently
	if OwnerLast.Move + AidPotWaitAfterMove >= ticks then
		-- Not time to use aid pot yet
		return castAllowed
	end

	-- Check if we're going to use a blue potion
	local level
	if not blue then
		level = AidPotLevel
		-- Check our HP
		if ActorHP(HomunID) >= AidPotHP then
			-- Not time to use aid pot yet
			return castAllowed
		end
	else
		level = 5
		-- Check our SP
		if ActorSP(HomunID) >= AidBlueSP then
			-- Not time to use aid pot yet
			return castAllowed
		end
	end

	-- Get the amount of SP that we have
	if SPUsable(OwnerID) < 1 then
		-- We don't have enough SP
		return castAllowed
	end

	-- Check if they're in range to get aid-potted
	if AADistance(HomunID,OwnerID) > 10 and not HoldPosition then
		-- Try to move closer
		AidPot_MoveCloser()

		-- They're out of range
		return castAllowed
	end

	-- Check if we're already casting something
	--	(ignoring timeouts set by ourselves)
	if (OwnerMotion ~= MOTION_CASTING or OwnerSkillTimeout ~= 0) and OwnerMotion ~= MOTION_SKILL then
		-- Log it
		Log(4,"Using Aid Potion, level %d.",level)

		-- Use the skill
		SkillObject(HomunID, level, 231, HomunID)

		-- Set the timeout
		if OwnerSkillTimeout < ticks + AidPotWaitTime then
			OwnerSkillTimeout = ticks+AidPotWaitTime
		end
	end

	-- Don't allow auto-bolting while we're aid potting
	return false
end

function AI_AidPotion_Blue(castAllowed)
	-- Reuse the basic code, as blue potions are almost the same
	return AI_AidPotion_Basic(castAllowed,true)
end

-- Function to check (and use) Aid Potion if necessary, with advanced checks
function AI_AidPotion_Adv(castAllowed)
	--[[ NOTE: If castAllowed is false, it will only check to see
		if a previous Aid Potion was successful
	--]]

	-- Get the tick count
	local ticks = GetTick()

	-- Check if we've had a success yet
	local success = false
	if AidPotAttempted > 0 then
		-- Check if our HP delta is positive (or owner's motion is PP)
		if HomunHPDelta.iteration > 0 or OwnerMotion == MOTION_PP then
			-- It was, so we probably were successful
			success = true
		else
			-- Our HP delta wasn't positive, so check if
			-- it was better than the last few
			
			-- Loop through the history of HP deltas
			local i,count,hpD = 0,0,HomunHPDelta.iteration
			for i=AidPotHistory.last-1,AidPotHistory.first,-1 do
				-- Get the history entry
				local t = AidPotHistory[i]
				
				-- Check if the history was aid-potted
				if not t[2] then
					-- It wasn't, check if we beat it
					if t[1] >= hpD then
						-- It wasn't better, so break
						break
					end
					
					-- Otherwise, add one to our count
					count = count + 1
				end
			end
			
			-- See if we beat the last 3 or more (that weren't aided)
			if count >= 3 then
				-- Count it as a success
				success = true
			end
		end

		-- Check if we found success
		if success then
			-- Log it
			Log(0,"Aid Potion was apparently successful.")
				
			-- Reset the tries counter (we obviously haven't run out)
			AidPotTriesLeft[AidPotAttempted] = AidPotTriesPerLevel

			-- We'll count it as a success
			AidPotAttempted = 0

			-- Reset our timeout as well
			AidPotTimeout = 0
		end
	end
	
	-- Add the HP delta to our history list
	List.pushright(AidPotHistory,{ HomunHPDelta.iteration, success })
	
	-- Trim it down if its too long
	while List.size(AidPotHistory) > 40 do
		List.popleft(AidPotHistory)
	end

	-- Check if our owner is still waiting for cast delay
	if ticks < AidPotTimeout and AidPotTimeout ~= 0 then
		-- Still waiting
		return castAllowed
	end

	-- Check if we failed (attempted was never reset to 0)
	if AidPotAttempted > 0 then
		-- Subtract a try from that level
		AidPotTriesLeft[AidPotAttempted] = AidPotTriesLeft[AidPotAttempted] - 1

		-- Log it
		Log(0,"Aid Potion appears to have been unsuccessful, %d tries left for Aid Potion level %d",
			AidPotTriesLeft[AidPotAttempted],AidPotAttempted)
		
		-- Set attempted to 0
		AidPotAttempted = 0
	end

	-- If we're not allowed to cast...
	if not castAllowed then
		-- ... Then we'll pass it on
		return castAllowed
	end

	-- Check if our owner is sitting down.
	if OwnerMotion == MOTION_SIT then
		-- We can't do anything :(
		return castAllowed
	end

	-- Check if our owner walked recently
	if OwnerLast.Move + AidPotWaitAfterMove >= ticks then
		-- Not time to use aid pot yet
		return castAllowed
	end

	-- Check our HP
	if ActorHP(HomunID) >= AidPotHP then
		-- Not time to use aid pot yet
		return castAllowed
	end

	-- Get the amount of SP that we have
	if SPUsable(OwnerID) < 1 then
		-- We don't have enough SP
		return castAllowed
	end

	-- Seems like its time to use it, check the number
	-- of attackers of us

	local enemies = ActorEnemies(HomunID)
	local numEnemies = Table.getn(enemies)

	-- Determine the level of aid potion to use
	local level = math.floor(numEnemies / AidPotIncrement)

	-- Check if its time to use it
	if level == 0 then
		-- Not time to use aid pot yet
		return castAllowed
	end
	
	-- Add our lowest level (so it really is increment)
	level = level + AidPotLowest - 1

	-- Make sure its not higher than our highest
	if level > AidPotHighest then
		-- Set it to highest
		level = AidPotHighest
	end

	-- Make sure we have the potions required
	local start,highest = level,AidPotHighest
	while level <= highest do
		-- Check if we have potions of this level
		if AidPotTriesLeft[level] > 0 then
			-- We do, so try to use 'em
			break
		end
		
		-- Check if incrementing would put us past our highest
		if level + 1 > AidPotHighest then
			-- Check if we can start from the bottom
			if start ~= AidPotLowest then
				-- Set our new highest
				highest = start - 1
				
				-- Set our new level
				level = AidPotLowest
				
				-- Set our new start
				start = level
			else
				-- We can't, just exit the loop
				break
			end
		else
			-- Otherwise, increment the potion level by 1
			level = level + 1
		end
	end

	-- Check if we have any potions
	if AidPotTriesLeft[level] > 0 then
		-- Check if they're in range to get aid-potted
		--	NOTE: This used to try to move closer, even when no
		--		potions were available.
		if AADistance(HomunID,OwnerID) > 10 and not HoldPosition then
			-- Try to move closer
			AidPot_MoveCloser()

			-- They're out of range
			return castAllowed
		end

		-- Check if we're already casting something
		if OwnerMotion ~= MOTION_CASTING and OwnerMotion ~= MOTION_SKILL then
			-- Log it
			Log(0,"Attempting to automatically use Aid Potion, level %d.",
				level)

			-- Use the skill
			SkillObject(HomunID, level, 231, HomunID)

			-- Set the timeouts (cast delay)
			OwnerSkillTimeout = ticks+AidPotWaitTime
			AidPotTimeout = OwnerSkillTimeout

			-- Reset the success variable
			AidPotAttempted = level
		end

		-- Don't allow auto-bolting while we're aid potting
		return false
	end

	-- Since we're not aid-potting, allow bolts to fire
	return castAllowed
end

-----------------------
-- Auto-Support Heal --
-----------------------

function AI_AutoSupportHealCheck(castAllowed)
	-- Check if we tried to use heal before
	if not SupportHealInfo.Active then
		-- We didn't
		return castAllowed
	end

	-- Get our sp
	local sp = SPUsable(OwnerID)

	-- Get the ticks
	local ticks = GetTick()

	-- Check if the alchemist is using a skill towards our target
	if OwnerMotion == MOTION_SKILL
		-- (target doesn't get updated with MOTION_SKILL :(
		-- and GetV(V_TARGET,OwnerID) == SupportHealInfo.Target
		and SupportHealInfo.SP - 10 + (3*SupportHealInfo.Level) >= sp
		then

		-- Log it
		Log(3,"Heal was apparently successful.")

		-- Set the owner skill timeout
		OwnerSkillTimeout = SupportHealInfo.Ticks + 1000

		-- Increment the number of heals against the target
		ActorData[SupportHealInfo.Target].Count.Heal =
			ActorData[SupportHealInfo.Target].Count.Heal + 1

		-- Since it was successful, unblock heals
		SupportHealInfo.Active = false

	else

		-- Check if it timed out
		if SupportHealInfo.Ticks + AutoAlchSkillDelay < ticks then
			-- Log it
			Log(3,"Heal apparently failed.")

			-- Unset the active flag
			SupportHealInfo.Active = false
		end

	end

	-- Pass "castAllowed" through
	return castAllowed
end

function AI_AutoSupportHeal(castAllowed,table,name,hitsPerHeal)
	-- Check if a cast is allowed
	if not castAllowed then
		-- No cast allowed
		return castAllowed
	end

	-- Check if another function tried to support heal
	if SupportHealInfo.Active then
		-- They already tried
		return castAllowed
	end

	-- Get the current ticks
	local ticks = GetTick()

	-- Try not to break autoattack (ctrl+click, or /nc and click)
	if OwnerLast.Attack + AutoAlchSkillDelay >= ticks then
		-- Too early to cast, they attacked recently
		return castAllowed
	end

	-- Check if we've moved recently
	if OwnerLast.Move + AutoAlchSkillDelay >= ticks then
		-- Too early to cast
		return castAllowed
	end

	-- Check to see if the alchemist is casting or using a skill
	if OwnerMotion == MOTION_CASTING or
		OwnerMotion == MOTION_SKILL then

		-- Can't cast right now
		return castAllowed
	end

	-- Find the level of healing to use
	local sp = SPUsable(OwnerID)
	local i,level = nil,0
	for i=3,1,-1 do
		-- Check if we have enough sp
		if sp >= 10 + (3*i) then
			-- Use the level
			level = i
			break
		end
	end

	-- Make sure we can use heal
	if level == 0 then
		-- We can't
		return castAllowed
	end

	-- Find the best person to heal
	local key,value,loopPairs = nil,nil
	local selected,selectHeals = 0,0
	for key,value in pairs(table) do
		-- Get the ID
		local id = key

		-- Check if they're within range and aren't a monster
		if AADistance(OwnerID,id) == nil then
			Log(0,"%s",tostring(id))
		end
		if AADistance(OwnerID,id) <= 10 and
			IsMonster(id) == 0 then

			-- Get their data table
			local t = ActorData[id]

			-- Check if they've been damaged recently
			if t.Last.Damage + AutoHealHitTimeouts < ticks then
				-- Up the number of heals
				t.Count.Heal = math.ceil(t.Count.Damage /
					hitsPerHeals)

				-- Up the number of hits, so the next will start
				--	counting against the number needed
				t.Count.Damage = t.Count.Heal * hitsPerHeals

			else

				-- Grab the number of heals needed to be done
				local healsNeeded = math.floor(t.Count.Damage /
					hitsPerHeals) - t.Count.Heal

				-- Check if they're more than the current
				if healsNeeded > selectHeals then
					-- Select this target for heals
					selected = id
					selectHeals = healsNeeded
				end
			end

		else
			-- Since they're out of range, we won't heal them
			ActorData[id].Count.Heal =
				math.ceil(ActorData[id].Count.Damage / hitsPerHeals)
		end
	end

	-- Make sure we have a player selected
	if selected == 0 then
		-- We don't
		return castAllowed
	end

	-- Log it
	Log(2,"Attempting to cast heal (level %d) on %s #%d",level,name,selected)

	-- Try to cast
	SkillObject(HomunID, level, 28, selected)

	-- Set the info for later use
	SupportHealInfo.Active = true
	SupportHealInfo.Ticks = ticks
	SupportHealInfo.Target = selected
	SupportHealInfo.Level = level
	SupportHealInfo.SP = sp
	SupportHealInfo.DmgCount = ActorData[selected].Count.Damage

	-- Allow other skills to try, in case we don't have light epsilon equipped
	return castAllowed
end

---------------------------------
-- Auto-Alchemist Attack Skill --
---------------------------------

-- Function to refund some time to a mutually exclusive skill
function AI_AutoAlchAttack_RefundTime()
	local ticks = GetTick()
	local EvalDelta = ticks - AlchAttackInfoHash.Any.LastEvalTime
	if EvalDelta ~= ticks then
		AlchAttackInfoHash.Any.NextMutExTime = AlchAttackInfoHash.Any.NextMutExTime + EvalDelta
	end

	-- Set the last time we evaluated mutually exclusive time
	AlchAttackInfoHash.Any.LastEvalTime = ticks

end

-- Function to check if we should use certain skills
function AI_AutoAlchAttack(castAllowed,skillName,skillIDs,spTable,range,minLevel,maxLevel,
	timeout,castTimeBase,castTimePerLevel,castDelayBase,castDelayPerLevel,successMotion,
	target,splashRange,minTargets,mutuallyExclu,testOnly,attemptReturn)


	-- Hidden Option -> Time before opening mutually exclusive to any of them
	local TimeBetweenMutExes = 1000
	-- Hidden Option -> Number of failure before opening other mutually exclusive skills
	local FailsBetweenMutExes = 1

	-- Check if we're sitting down
	if OwnerMotion == MOTION_SIT then
		-- Can't do anything :(
		return castAllowed
	end

	-- Check to see if we're allowed to cast
	if not castAllowed then
		return castAllowed
	end

	-- Get the current ticks
	local ticks = GetTick()

	-- Decide if a mutually excluse skill is active
	local mutexActive = false
	if mutuallyExclu and ticks < AlchAttackInfoHash.Any.NextMutExTime then
		mutexActive = true
	end

	-- Get the attack info table from the hash
 	if AlchAttackInfoHash[skillName] == nil then
		-- Create one if it didn't exist
		AlchAttackInfoHash[skillName] = {
			Active = false,
			TimeBase = 0,
			TimePerLevel = 0,
			DelayBase = 0,
			DelayPerLevel = 0,
			LevelAttempted = 0,
			Failed = 0,
			TempDisabled = false,
		}
	end
	local AlchAttackInfo = AlchAttackInfoHash[skillName]

	-- Get the amount of SP we're allowed to use
	local sp = SPUsable(OwnerID)

	-- Check if we previously tried casting these skills
	if AlchAttackInfoHash.Any.Active and AlchAttackInfo.Active and MaxDistance == 0 and castAllowed then

		-- Check if we're casting
		if OwnerMotion == successMotion then
			-- We're casting now, set our timeout

			-- Get the cast time
			local castTime = CastModifier * (castTimeBase + (castTimePerLevel * AlchAttackInfo.LevelAttempted))

			OwnerSkillTimeout = ticks +
				-- Cast Time
				castTime +

				-- Cast Delay
				castDelayBase + (castDelayPerLevel * AlchAttackInfo.LevelAttempted)

			local used = "Using"
			if castTime == 0 then
				used = "Used"
			end

			local mutexPost = ""
			if mutuallyExclu and ticks >= AlchAttackInfoHash.Any.NextMutExTime then
				mutexPost = "(?)"
			end

			-- Log it
			Log(2,"%s alchemist skill (%s%s; level: %d) against Actor #%d.",used,skillName,mutexPost,
				AlchAttackInfo.LevelAttempted,AlchAttackInfo.Target)

			-- Set the time which we last casted a skill (or rather, when the skill will complete)
			LastAlchAttackSkill = OwnerSkillTimeout

			-- Reset all AlchAttackInfo active flags
			AlchAttackInfoHash.Any.Active = false
			AlchAttackInfo.Active = false
			AlchAttackInfo.Failed = 0
			if mutuallyExclu then
				AlchAttackInfoHash.Any.LastMutEx = skillName
				AlchAttackInfoHash.Any.NextMutExTime = LastAlchAttackSkill + TimeBetweenMutExes + timeout + AutoAlchSkillDelay * FailsBetweenMutExes
			end

			-- Set the last time we evaluated mutually exclusive time
			AlchAttackInfoHash.Any.LastEvalTime = ticks

			-- Don't allow casting, but tell our caller that they should increment
			--	the skill usage counter
			return false,true

		-- Check if it timed out
		elseif ticks > AlchAttackInfo.Timeout then

			-- Reset AlchAttackInfo data
			AlchAttackInfo.Active = false

			-- Check if a mutually exclusive skill is active
			if mutexActive then

				-- Check if we're the one
				if AlchAttackInfoHash.Any.LastMutEx == skillName then

					-- Increment the number of failed tries
					AlchAttackInfo.Failed = AlchAttackInfo.Failed + 1

					-- Check if we've had failed a few tries
					if AlchAttackInfo.Failed >= FailsBetweenMutExes then
						-- Reset the failed tries, and allow a different
						--	mutually exclusive ability to try
						AlchAttackInfo.Failed = 0
						AlchAttackInfo.TempDisabled = true
						AlchAttackInfoHash.Any.NextMutExTime = 0
						AlchAttackInfoHash.Any.DisableUntil = ticks + TimeBetweenMutExes + timeout + AutoAlchSkillDelay * FailsBetweenMutExes
						return castAllowed
					end

					-- We should refund some time
					AI_AutoAlchAttack_RefundTime()

				else

					-- Set the last evaluated time
					AlchAttackInfoHash.Any.LastEvalTime = ticks

				end
			end

		end
	end

	-- Check to see if we were just using the above test
	if testOnly then
		return castAllowed
	end

	-- Check to see if we're temporarily disabled
	if AlchAttackInfo.TempDisabled then
		if AlchAttackInfoHash.Any.DisableUntil > ticks then
			-- We're disabled
			return castAllowed
		else
			-- We shouldn't be disabled any longer
			AlchAttackInfo.TempDisabled = false
		end
	end

	-- Check if its too early to cast against this monster
	if LastAlchAttackSkill + timeout > ticks then
		-- Too early to cast against it
		return castAllowed
	end

	-- Check to see if we're an active mutually exclusive skill while they're locked
	if mutexActive then

		-- Check if we are the mutually exclusive skill
		if AlchAttackInfoHash.Any.LastMutEx ~= skillName then
			-- We're not the active mutually exclusive skill
			return castAllowed
		else
			-- Check if we should extend the time that we're mutually exclusive
			if AlchAttackInfoHash.Any.LastEvalTime ~= 0 and
				(MaxDistance ~= 0 or target == -1 or
				OwnerLast.Attack + AutoAlchSkillDelay >= ticks or
				OwnerLast.Move + AutoAlchSkillDelay >= ticks) then

				-- We should refund some time
				AI_AutoAlchAttack_RefundTime()

				-- We should return now
				return castAllowed

			end

		end
	else

		-- Don't evaluate this following stuff twice in event of an active mutually exclusive skill

		-- Check if we're still in a delay time for auto-res
		if MaxDistance ~= 0 then
			return castAllowed
		end

		-- Check if we have a target for our alchemist
		if target == -1 then
			return castAllowed
		end
	
		-- Try not to break autoattack (ctrl+click, or /nc and click)
		if OwnerLast.Attack + AutoAlchSkillDelay >= ticks then
			-- Too early to cast, they attacked recently
			return castAllowed
		end
	
		-- Check if we've moved recently
		if OwnerLast.Move + AutoAlchSkillDelay >= ticks then
			-- Too early to cast
			return castAllowed
		end
	
	end

	-- Check if we're already casting
	if	OwnerMotion == MOTION_SKILL or
		OwnerMotion == MOTION_CASTING then

		-- We don't want to cast right now
		return castAllowed

	end

	-- Get the location of our target
	local x,y = GetV(V_POSITION,target)

	-- Check if they're within range
	if APDistance(OwnerID,x,y) > range then
		-- Out of range :(
		return castAllowed
	end

	-- See if the skill is a splash-damage skill
	if splashRange > 0 then
		-- Loop through each actor, to see if its in range
		local actor,dummy,count = nil,nil,0
		for actor,dummy in pairs(ActorData) do
			-- Make sure its not the homunculus or owner
			if actor ~= HomunID and actor ~= OwnerID then
				-- Check if the actor is a valid target for splash
				if dummy.Motion ~= MOTION_DEAD and
					not IsFriend(actor) and
					IsMonster(actor) == 1 and

					-- The following are disabled because the
					--	monster will be hit no matter what
					-- MonsterAllowed(actor,true) and
					-- not IsIgnored(actor) and
					-- not KillSteal(actor) and

					APDistance(actor,x,y) <= splashRange then

					-- Add to the count
					count = count + 1

				end


			end
		end

		-- Check if we have enough monsters that'll be splashed
		if count < minTargets then
			-- Not enough targets
			return castAllowed
		end
	end

	-- Get the max level we can use against our target
	local max = maxLevel

	-- Make sure the maximum level is a sane value
	if max < 1 then
		-- Don't use the skill
		return castAllowed
	elseif max > Table.getn(spTable) then
		-- Set the max level
		max = Table.getn(spTable)
	end

	-- Get the minimum level we should cast of our skill
	local min = minLevel

	-- Make sure max isn't below min
	if max < min then
		min = max
	end

	-- Determine which level to use
	local level = 0
	for i=max,min,-1 do
		if sp >= spTable[i] then
			level = i
			break
		end
	end

	-- Check if we are going to use a valid level
	if level == 0 then
		-- Refund time to a mutexed skill
		if mutexActive then
			AI_AutoAlchAttack_RefundTime()
		end

		-- Don't cast a skill
		return castAllowed
	end

	-- Modify the target from OwnerID to HomunID
	--	(skills that target the owner will never work,
	--	so we fudge it and use the homun. It usually
	--	works just as well)
	if target == OwnerID then
		target = HomunID
	end

	-- Use the skills
	for i=1,Table.getn(skillIDs) do
		SkillObject(HomunID, level, skillIDs[i], target)
	end

	-- Set the data for later
	AlchAttackInfoHash.Any.Active = true
	AlchAttackInfo.Active = true
	AlchAttackInfo.LevelAttempted = level
	AlchAttackInfo.Target = target
	AlchAttackInfo.Timeout = ticks + AutoAlchSkillDelay

	-- Set the last time we evaluated mutually exclusive time
	if mutexActive then
		AlchAttackInfoHash.Any.LastEvalTime = ticks
	end

	-- Return that we're attempting to use the skill
	return attemptReturn
end


------------------------
-- Auto-Attack Skills --
------------------------

-- Function to use a bolt-sword (Ice Falchion/Fireblend)
--		(idea from a variety of people)
function AI_AutoBolt(castAllowed)
	-- Make sure we have a target
	if AlchTarget.ID == -1 then
		return castAllowed
	end

	-- Get the actor data table
	local data = ActorData[AlchTarget.ID]

	-- Check if we've casted too many times against this one
	local testOnly = false
	if MAlchSkMaxCasts(AlchTarget.ID) <= data.Count.AlchSkill then
		-- Refund time if we're mutexed
		if GetTick() < AlchAttackInfoHash.Any.NextMutExTime and
			AlchAttackInfoHash.Any.LastMutEx == "Sword-bolt" then
			AI_AutoAlchAttack_RefundTime()
		end

		-- Don't cast
		testOnly = true
	end

	-- Reuse above code
	local ret,inc = AI_AutoAlchAttack(castAllowed,
		-- Name and any possible skill IDs
		"Sword-bolt",{14,19},
		-- SP cost, by level
		{12,14,16},
		-- Range
		10,
		-- Mix/max levels
		AutoAlchSkillMinLevel,MAlchSkMaxLevel(AlchTarget.ID),
		-- Timeout between casts
		MAlchSkTimeout(AlchTarget.ID),
		-- Cast time/cast delay calculation inputs (base time, time/level, base delay, delay/level)
		0,700,800,200,
		-- Successful motion
		MOTION_CASTING,
		-- Target
		AlchTarget.ID,
		-- Splash Range (0 = non-splash)
		0,
		-- Minimum number of estimated monsters hit by splash before use
		1,
		-- Is this skill mutually exclusive to others marked true?
		--	(basically, if you're casting "ice bolt", you will never
		--	be able to cast "tomahawk throwing")
		true,
		-- Do not disable the cast attempt
		--	(true = test for a previous successful cast, do not attempt new cast)
		testOnly,
		-- On attempt, continue to allow other skills to attempt
		castAllowed)

	-- Check if the cast was successful
	if inc then
		data.Count.AlchSkill = data.Count.AlchSkill + 1
	end

	return ret
end

-- Function to use a light-epsilon to heal undead monsters
function AI_AutoHealBomb(castAllowed)
	-- Make sure we have a target
	if AlchTarget.ID == -1 then
		return castAllowed
	end

	-- Get the actor data table
	local data = ActorData[AlchTarget.ID]

	-- Check to see if the target is heal-bombable
	local testOnly = false
	if not MAlchSkHealBomb(AlchTarget.ID) then
		testOnly = true
	end

	-- Check if we've casted too many times against this one
	if not testOnly and MAlchSkMaxCasts(AlchTarget.ID) <= data.Count.AlchSkill then
		-- Refund time if we're mutexed
		if GetTick() < AlchAttackInfoHash.Any.NextMutExTime and
			AlchAttackInfoHash.Any.LastMutEx == "Heal" then
			AI_AutoAlchAttack_RefundTime()
		end

		-- Don't cast
		testOnly = true
	end

	-- Reuse above code
	local ret,inc = AI_AutoAlchAttack(castAllowed,
		"Heal",{28},
		{13,16,19},
		10,
		AutoAlchSkillMinLevel,MAlchSkMaxLevel(AlchTarget.ID),
		MAlchSkTimeout(AlchTarget.ID),
		0,0,1000,0,
		MOTION_SKILL,
		AlchTarget.ID,
		0,
		1,
		true,
		testOnly,
		castAllowed)

	-- Check if the cast was successful
	if inc then
		data.Count.AlchSkill = data.Count.AlchSkill + 1
	end

	return ret
end

-- Function to throw a tomahawk
function AI_ThrowTomahawk(castAllowed)
	-- Make sure we have a target
	if AlchTarget.ID == -1 then
		return castAllowed
	end

	-- Get the actor data table
	local data = ActorData[AlchTarget.ID]

	-- Check if we've casted too many times against this one
	local testOnly = false
	if MAlchSkMaxCasts(AlchTarget.ID) <= data.Count.AlchSkill then
		-- Refund time if we're mutexed
		if GetTick() < AlchAttackInfoHash.Any.NextMutExTime and
			AlchAttackInfoHash.Any.LastMutEx == "Tomahawk" then
			AI_AutoAlchAttack_RefundTime()
		end

		-- Don't cast
		testOnly = true
	end

	-- ID 337, sp 1, range 10?, cast motion?
	local ret,inc = AI_AutoAlchAttack(castAllowed,
		"Tomahawk",{337},
		{1},
		10,
		1,1,
		MAlchSkTimeout(AlchTarget.ID),
		0,0,100,0,
		MOTION_SKILL,
		AlchTarget.ID,
		0,
		1,
		true,
		testOnly,
		castAllowed)

	-- Check if the cast was successful
	if inc then
		data.Count.AlchSkill = data.Count.AlchSkill + 1
	end

	return ret
end

-- Function to bash
function AI_AutoBash(castAllowed)
	-- Make sure we have a target
	if AlchTarget.ID == -1 then
		return castAllowed
	end

	-- Get the actor data table
	local data = ActorData[AlchTarget.ID]

	-- Check if we've casted too many times against this one
	local testOnly = false
	if MAlchSkMaxCasts(AlchTarget.ID) <= data.Count.AlchSkill then
		-- Refund time if we're mutexed
		if GetTick() < AlchAttackInfoHash.Any.NextMutExTime and
			AlchAttackInfoHash.Any.LastMutEx == "Bash" then
			AI_AutoAlchAttack_RefundTime()
		end

		-- Don't cast
		testOnly = true
	end

	local ret,inc = AI_AutoAlchAttack(castAllowed,
		"Bash",{5},
		{8,8,8,8,8},
		2,
		AutoAlchSkillMinLevel,MAlchSkMaxLevel(AlchTarget.ID),
		MAlchSkTimeout(AlchTarget.ID),
		0,0,100,0,
		MOTION_SKILL,
		AlchTarget.ID,
		0,
		1,
		true,
		testOnly,
		castAllowed)

	-- Check if the cast was successful
	if inc then
		data.Count.AlchSkill = data.Count.AlchSkill + 1
	end

	return ret
end

-- Function to mammonite
function AI_AutoMammo(castAllowed)
	-- Make sure we have a target
	if AlchTarget.ID == -1 then
		return castAllowed
	end

	-- Get the actor data table
	local data = ActorData[AlchTarget.ID]

	-- Check if we've casted too many times against this one
	local testOnly = false
	if MAlchSkMammo(AlchTarget.ID) <= data.Count.AlchMammo then
		-- Don't cast
		testOnly = true
	end

	local ret,inc = AI_AutoAlchAttack(castAllowed,
		"Mammo",{42},
		{5,5,5,5,5,5,5,5,5,5},
		2,
		10,10,
		MAlchSkTimeout(AlchTarget.ID),
		0,0,100,0,
		MOTION_SKILL,
		AlchTarget.ID,
		0,
		1,
		false,
		testOnly,
		false)

	-- Check if the cast was successful
	if inc then
		data.Count.AlchMammo = data.Count.AlchMammo + 1
	end

	return ret
end

-- Function to acid terror
function AI_AutoAcidTerror(castAllowed)
	-- Make sure we have a target
	if AlchTarget.ID == -1 then
		return castAllowed
	end

	-- Get the actor data table
	local data = ActorData[AlchTarget.ID]

	-- Check if we've casted too many times against this one
	local testOnly = false
	if MAlchSkAcidTerror(AlchTarget.ID) <= data.Count.AlchAcidT then
		-- Don't cast
		testOnly = true
	end

	-- ID 230

	local ret,inc = AI_AutoAlchAttack(castAllowed,
		"AcidTerror",{230},
		{15,15,15,15,15},
		10,
		5,5,
		MAlchSkTimeout(AlchTarget.ID),
		1000,0,500,0,
		MOTION_SKILL,
		AlchTarget.ID,
		0,
		1,
		false,
		testOnly,
		false)

	-- Check if the cast was successful
	if inc then
		data.Count.AlchAcidT = data.Count.AlchAcidT + 1
	end

	return ret
end

-- Function to acid bomb
function AI_AutoAcidBomb(castAllowed)
	-- Make sure we have a target
	if AlchTarget.ID == -1 then
		return castAllowed
	end

	-- Get the actor data table
	local data = ActorData[AlchTarget.ID]

	-- Check if we've casted too many times against this one
	local testOnly = false
	if MAlchSkAcidBomb(AlchTarget.ID) <= data.Count.AlchAcidB then
		-- Don't cast
		testOnly = true
	end

	-- ID 490, sp 30 (all levels), 1 sec cast time (all levels), 1 sec cast delay (all levels), MOTION_BIGTOSS

	local ret,inc = AI_AutoAlchAttack(castAllowed,
		"AcidBomb",{490},
		{30,30,30,30,30,30,30,30,30,30},
		10,
		10,10,
		MAlchSkTimeout(AlchTarget.ID),
		1000,0,1000,0,
		MOTION_BIGTOSS,
		AlchTarget.ID,
		0,
		1,
		false,
		testOnly,
		false)

	-- Check if the cast was successful
	if inc then
		data.Count.AlchAcidB = data.Count.AlchAcidB + 1
	end

	return ret
end

-- Function to cart revolution
function AI_CartRevo(castAllowed,skillID,spTable,min,max,target)
	-- Make sure we have a target
	if AlchTarget.ID == -1 then
		return castAllowed
	end

	return AI_AutoAlchAttack(castAllowed,
		"CartRevo",{153},
		{12},
		2,
		1,1,
		AutoSplashTimeout,
		0,0,100,0,
		MOTION_SKILL,
		AlchTarget.ID,
		2,
		AutoSplashMinHits,
		false,
		false,
		false)
end

-- Function to magnum break
function AI_MagnumBreak(castAllowed)
	return AI_AutoAlchAttack(castAllowed,
		"MagBreak",{7},
		{15,15,15},
		2,
		AutoSplashBlastMinLvl,3,
		AutoSplashTimeout,
		0,0,2000,0,
		MOTION_SKILL,
		OwnerID,
		2,
		AutoSplashMinHits,
		false,
		false,
		false)
end
