--[[

This file is part of RampageAI.

RampageAI is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

RampageAI is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with RampageAI; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

--]]

--[[
*** Readme ***
Any line beginning with "--" is a "comment line" and has no effect.
Also, "--[[" is the beginning of a "comment block" and no lines will
have any effect until a "--]]" is reached.

The confurable options all appear on their own lines with a single "="

Example:

ThisOptionIsInACommentBlock = true


Because you do not need any knowledge of programming to change numbers,
do not send any support or help requests with "I do not know lua",
"I do not know programming", or other such statements. Doing so will tell
me (Faithful) that you have not read your configuration file, and I may
ignore your request.

For this reason, I will once again go over >>everything<< that you need to
know. All options follow the format as follows:

OptionName = OptionValue

Never, ever change the OptionName. Never. But, if you do not like the setting
which is dictated, then you may change OptionValue. Do not worry if you
get it wrong. The script checks every option to make sure it has a valid value.
If you somehow do get something wrong, it will return it to the default to prevent
Ragnarok from giving a lua-error box.

As an example, I will show how to make the homunculus run right next to you
every time you get too far away.

The option that has this setting is shown below as follows:
HomunFollowDistance = 3
But we want the homunculus to come next to us, so we change it as follows:
HomunFollowDistance = 1

Now relog, and the homunculus will chase right up next to us when we run away.

Thats all you need to know! :)
--]]

-------------------
-- Debug Options --
-------------------

--[[ Enable/disable debugging through Log()

	Disabling this could be useful when attempting to debug your own
	AddOns using TraceAI()

	NOTE: You will also have to type /traceai in the RO client
--]]
Debug = true

-- How verbose do you want the log to be?
--	(0 is the least verbose, each increment past that makes it more verbose)
DebugLevel = 2

-- Work-around for Log() when TraceAI.txt only outputs the very
-- last TraceAI() command
SingleLineLogs = false

-- Maximum number of lines to output (only when SingleLineLogs is true)
LogLines = 30

-- Mark the log every X number of milliseconds (0 disables)
DebugMark = 20000

--[[
	Should we attempt to determine how much network usage RampageAI is using?

	NOTE: This does not "hack". This does not attempt to intercept packets.
		This does not inject any packets, and does not modify the
		ragnarok client.

	This ONLY attempts to estimate how much outgoing bandwidth RampageAI generates.

	See NetUsageMonitor.lua for the FULL code if you do not trust it.

	Alternatively, if you still do not trust, it then leave it disabled.

--]]
EnableNetUsageMonitor = false


---------------------------
-- Alchemist Information --
---------------------------

-- Total dex (Base + Bonus)
AlchemistDex = 30

-- Wearing a phen clip?
AlchemistPhen = false

----------------------
-- Distance Options --
----------------------

-- Anything beyond this number of tiles away from Owner is ignored
SightRange = 12			-- max=15 (not recommended)

-- Distance until Homunculus begins chasing owner
MaxHomunRange = 12		-- recommended=12

-- Distance that the MaxHomunRange is reduced by if the owner is moving
--	away from the homunculus
HomunRangeChange = 4		-- recommended=4

-- Distance that the homunculus should aim for when following its owner
HomunFollowDistance = 3

--[[ Should the homunculus always try to stay ahead of its owner?

	NOTE: This tries to keep it ahead of the alchemist by the number
		specified in HomunFollowDistance
--]]
HomunRunAhead = false

--[[ Allow the homunculus to ignore sequences to follow its master
	in the event that it gets stuck behind something?

	NOTE: Disabling this may cause the script to get "stuck" when following
		the alchemist.
--]]
AllowIgnoreOwner = true

-- Farthest distance that the homunculus is allowed to move with 1 command
--	(lower numbers will make shorter moves, and help with lag)
HomunMaxMoveDistance = 12	-- max of 12, server ignores any higher

-------------------
-- Idle Movement --
-------------------

-- Do you want the homunculus to move around when it is not doing anything else?
EnableIdleMove = true

-- Circle radius (must be between 2 and 6, inclusive)
IdleMoveRadius = 5

-- How many ticks (milliseconds) should pass before the idle movement begins
TimeBeforeIdle = 1000

-- Start moving around only after the homunculus's HP and SP are above
-- these amounts/percents
IdleMoveHP = 50
IdleMoveSP = 50

--------------------------------
-- Advanced Movement Commands --
--------------------------------

-- See ..\Docs\AdvMove.txt for details

-- Allow Advanced Movement Commands to be processed
EnableAMC = true

-- Use "Quickdial" AMCs instead
QuickdialAMC = true

-- If no advanced movement command is received in X milliseconds, clear
-- the command sequence so far.
AMCResetTime = 5000

--------------------
-- Attack Options --
--------------------

-- Default to being aggressive on map-change/flywing/call homun/etc
DefaultAggro = false

-- Assist the homunculus's owner?
AssistOwner = true

-- Allow "kill stealing"
AllowKS = false

--[[ On occassion, the homunculus will attempt to attack a monster that is
	on the other side of a wall, but the script will recognize this and
	ignore that monster for a number of milliseconds. This option sets that number.
	
	0 will disable.
	
	NOTE: This option will only be effective in multiples of 500
--]]
ActorIgnoreTime = 5000

-- Should we ignore a monster if it takes too long to kill it?
EnableIgnoreAfterTicks = false

-- How long should we wait before ignoring it? (minimum 5 seconds. 1 tick = 1 millisecond)
ActorIgnoreAfterTicks = 30000

-- Check for a new target only once every this many milliseconds
AcquireTargetTimeout = 50

--[[ Allow the AI to select a different target, even when it is locked onto a monster?

	NOTE: This will allow it to change to a higher priority monster if one appears,
		and it might also start to defend the alchemist sooner.

	NOTE: This will affect both the alchemist and the homunculus

	WARNING: This may cause unexpected behaviour.
--]]
AcquireWhileLocked = false

--[[
	Sometimes the monster that the homunculus is attacking will get "position lagged"
	near to where the homunculus is. Because of this, the homunculus will send attacks
	but they will never happen (because the homunculus is actually too far away!)

	RampageAI will detect this and attempt to move to a different spot in order to
	fix this position lag.

	NOTE: Because RampageAI cannot detect when the homunculus attacks, it uses
		the times that a monster takes damage to determine if its attacks are
		working. Therefore, if you use this and your homunculus has low HIT,
		it will think that its attacks are not working and attempt to move
		to a different location.

--]]
EnableAttacksTimeout = true		-- enabled?
AttacksTimeout = 1500			-- how long since last attack before moving?
AttacksMaxTries = 5			-- how many tries before checking the time?

--------------------
-- Kiting Options --
--------------------

-- Please look inside Monsters.lua to set which monster types to kite.

-- How far ahead of the monster should we try to stay
KiteDefaultDistance = 6

-------------------------------
-- Passive/Defensive Options --
-------------------------------

--[[ Allow target acquisition while in passive mode

	To be more specific, this will allow the homunculus to
	automatically begin attacking monsters that target itself,
	its owner, or any members of its friend list

	Also, if this is set to false, all the other options have
	no effect, unless otherwise noted
--]]
PassiveDefense = true

-- Allow automatic casting of attack-skills while in passive mode
PassiveSkills = true

--[[ The following 3 options deal with defense prioritization. Defense
	prioritization works in the following manner:

	1) Each of the 3 options specifies the number of monsters that need to be
	attacking a specific thing before that thing's defense gets prioritized.

	2) Friends will be checked individually against the PrioritizeFriendDefense
		number >ONLY< if the option DefendFriends (below) is true.

	3) If nothing meets the critera to have its defense prioritized, RampageAI will
		target the highest priority monster attacking anybody.

	4) If a single person meets the criteria to have their defense prioritized, then
		RampageAI will target the highest priority monster attacking that person.

	5) If more than 1 person meets the criteria, RampageAI will judge how many monsters
		was required to prioritize each person, and decide who to defend first.

		a) It will always try to defend the person that required the most number of
			attacking monsters first; however, if there is a tie, it will
			defend whichever person has more monsters attacking them.

	- Any of the 3 options may be set to 0 to disable them.

	Confused yet? If so, the defaults should be fine.

--]]
PrioritizeOwnerDefense = 1
PrioritizeHomunDefense = 5
PrioritizeFriendDefense = 4

--------------------
-- Friend Options --
--------------------

--[[ Defend friends from monsters as well?

	NOTE: this also allows KSing of friends, even when the AllowKS
		option is turned off
--]]
DefendFriends = true

--[[ Sometimes it may be beneficial to automatically add friends that sit/stand
	within a certain range of you.

	Rampage AI will temporarily add people as friends if they're within this
	many tiles from of you.

	(idea from i2apture of RagnaInfo forums)

	NOTE: Any negative number will disable it.
--]]
TemporaryFriendTiles = -1

-- Automatically save the friends list when it gets updated
AutoSaveFriends = false

---------------------------
-- General Skill Options --
---------------------------

--[[ Allow any automatic casting of skills

	NOTE: This will disable *all* homunculus skill usage if set to false
--]]
AllowSkills = true

-- Manual targetting of skills causes target to become enemy
SkillsCauseAttack = false

-- Do not use skills if SP will be lower this amount/percent after skill
MinSP = 0

--------------------------
-- Attack Skill Options --
--------------------------

-- Attack Skills consist of Vanilmirth's Caprice and Filir's Moonlight.

--[[ Because of the way homunculi skills work, the attack skills are
	actually ranged attacks that the alchemist uses. This allows us to
	use the skills at seemingly any range, even if they would be out of range
	for the homunculus.

	Set this option to true to take advantage of this bug.
--]]
IgnoreAttackSkillRange = false

-- If the above option is true, this option specifies how far from the alchemist
--	a monster can be to be inside range
IgnoredRangeFromOwner = 15

--[[ Minimum level of the attack skill to use

	The homunculus will always try to use the max level, but if it
	does not have enough SP, it will check each lower level.
	This option will limit it to never using an attack skill
	below a certain level.

	Set to 5 to always use the highest level allowed by Monsters.lua.
--]]
MinAttackSkillLevel = 1

-- Allow automatic casting of attack-skills while meleeing enemies
MeleeSkills = true

-- Allow automatic casting of attack-skills while chasing enemies
ChaseSkills = true

-- Lock onto a target as soon as an attack skill was automatically cast at it
--	(this may cause the script to get "stuck")
LockOnAttackSkills = false

-------------------------
-- Other Skill Options --
-------------------------

-- Allow automatic casting of support-skills (Amistr Bulwark, Flitting, etc)
SupportSkills = true

--[[ Allow automatic casting of heal-skills (Healing Hands and Chaotic Blessings)

	NOTE: When the homunculus is trying to heal, it will stop casting attack
		skills to ensure that it will get SP to heal with
--]]
HealSkills = true

---------------------------
-- Auto-Cast Alch Skills --
---------------------------

-- These options pertain to a few of the following sets of options

-- Allow the alchemist to use skills against monsters that the homunculus isn't attacking
AlchTargetSeparate = false

-- While targetting separately, should we specifically try to target different monsters?
--	(NOTE: AlchTargetSeparate must be set to true for this to work)
AlchTargetForceDifferent = false

-- When the alchemist's hp goes below this amount/percent, stop targeting separately
--	(0 = disabled)
AlchTargetSameWhenHP = 0

-- Minimum level of targetted attack skills to use
AutoAlchSkillMinLevel = 1

-- How quickly should the homunculus check to see if casting an alch skill worked
-- (in milliseconds)
AutoAlchSkillDelay = 200

------------------------------
-- Auto-Cast Splash Attacks --
------------------------------

-- Use Cart Revolution
AutoSplashCart = false

-- Use Magnum Break (blast clip)
AutoSplashBlast = false

-- Minimum level of magnum break to use
AutoSplashBlastMinLvl = 1

-- Minimum number of estimated monsters that'll be hit before using a splash attack
AutoSplashMinHits = 1

-- Time between using a splash attack (in milliseconds)
AutoSplashTimeout = 1000

--------------------
-- Auto-Cast Heal --
--------------------

-- Designed for use with light epsilon, not heal clip (heal clip is just too inefficient)

-- The number of hits that a friend must sustain before attempting to heal them
--	(0 = disabled)
AutoHealFriends = 0

-- The number of hits that a temporary friend must sustain before attempting heals at them
--	(0 = disabled, for temporary friends, see "TempFriendTiles" above in "Friend Options")
AutoHealTempFriends = 0

-- The number of hits that any player in range must take before getting healed by RampageAI
--	(0 = disabled)
AutoHealAnybody = 0

-- How recent should the latest damage be for an actor to be considered for healing?
--	(in milliseconds)
AutoHealHitTimeouts = 5000

-- Minimum level of healing to use when supporting another player
AutoHealMinLevel = 1

-- Attempt to "heal-bomb" undead monsters (specified in Monsters.lua option "hlBm")?
--	(NOTE: Uses "aSkDl", "aSkLv", "aSkMx", and "hlBm" from Monsters.lua table)
AutoHealBomb = false

--------------------------------
-- Other Auto-Castable Skills --
--------------------------------

-- Attempt to cast Acid Demonstration at people
--	(NOTE: Uses "aSkDl", and "aBmb" from Monster.lua tables)
AutoAcidDemo = false

-- Attempt to cast Acid Terror at people
--	(NOTE: Uses "aSkDl", and "aTer" from Monster.lua tables)
AutoAcidTerror = false

-- Attempt to make use of any of the bolt swords (Ice Falchion / Fireblend)
--	(NOTE: Uses "aSkLv", "aSkDl", and "aSkMx" from Monsters.lua tables)
-- *idea from a variety of people*
AutoBoltSword = false

-- Attempt to use Bash from Cutlus
--	(NOTE: Uses "aSkLv", "aSkDl", and "aSkMx" from Monsters.lua tables)
AutoCutlusBash = false

-- Attempt to use Axe Throwing from Tomahawk
--	(NOTE: Uses "aSkDl" and "aSkMx" from Monsters.lua tables)
AutoTomahawk = false

---------------------
-- Auto-Aid-Potion --
---------------------

-- Enable automatic use of aid potion
AutoAidPot = false

-- Use Aid Potion *only* when Homunculus's HP is below this amount or percent
-- NOTE: Check Misc. Options below for "PercentHP".
AidPotHP = 25

-- Level of aid potion to use
AidPotLevel = AIDPOT_WHITE

--[[ If the homunculus is outside the range of Aid Potion, but matches all
	of the criteria for a potion to be thrown at it, should it move
	closer to the alchemist?
--]]
AidPotMoveCloser = true

--[[ How much time should elapse after the alchemist moves before aid potion
	is allowed to throw potions? (0 to disable)
--]]
AidPotWaitAfterMove = 500

--------------------------
-- Auto-Aid-Blue Potion --
--------------------------

-- Idea from Fallen~Angel

-- Enable automatic potting with blues?
AutoAidBlue = false

-- Use Aid Blue *only* when Homunculus's SP is below this amount or percent
-- NOTE: Check Misc. Options below for "PercentSP".
AidBlueSP = 25

---------------------------------------
-- Auto-Aid-Potion Advanced Settings --
---------------------------------------

--[[ Use advanced settings?

		Use this if you want RampageAI to attempt to automatically detect when
		your alchemist runs out of potions, so that it can use the next level.
		It also attempts to use a higher level of potion when the homunculus
		gets more monsters on it.

		If you do not use advanced and your alchemist runs out of potions while
		trying to throw them, you will become stuck seeing infinite "skill failed"
		messages, until your homunculus either: (a) dies, or (b) regens enough hp.

	You have been warned.

--]]
AidPotAdvanced = false

-- Lowest "level" of potion to use
--	NOTE: This and AidPotHighest override AidPotLevel from above
AidPotLowest = AIDPOT_YELLOW

-- Highest "level" of potion to use
--	NOTE: This and AidPotLowest override AidPotLevel from above
AidPotHighest = AIDPOT_WHITE

--[[
	This option specifies how it'll increment the type of potion
	used, as follows:

	Less than X monsters attacking homunculus:	Do not use Aid Pot
	Between X and X * 2 - 1 monsters attacking:	Use lowest level of Aid Pot
	Between X * 2 and X * 3 - 1 monsters:		Use lowest + 1 Aid Pot
	Etc., etc.

	This option is "X"
	
	Here is an example to clarify for those who do not understand, as well as
	make this part long for those who choose not to actually read it:
	
	If you set this option to "3" it will work as follows:
	0 thru 2 monsters attacking homunculus ---> will *NOT* use aid potion
	3 thru 5 monsters attacking homunculus ---> will use the lowest level of aid potion
	6 thru 8 monsters attacking homunculus ---> will use the next level of aid potion
	
	I have given my best effort to describe everything about the way the script works
	in order to make aid potion configuration as simple as possible. The only way I
	would be able to better help you configure it the way you like it would be to
	directly tell you the settings to change, and I will probably not do that.
	
--]]
AidPotIncrement = 3

--[[
	The Aid-Potion script will automatically detect when the aid potion
	was not successful, but try again up to X number of times for each
	level. After it has failed X consequtive times on that level, it will
	try again on the next higher level.
	
	However, since aid potion can fail because the homunculus is around a
	corner (even if you have enough potions), the tries may be reset every
	Y number of milliseconds.
	
	Sometimes the connection to the server may lag, and the script will be
	unable to tell if the aid-potion was successful or not. The script
	watches the homunculus's HP to determine this, so if the HP does not
	show that it has been healed after Z milliseconds the script will
	assume the aid-potion has failed.
	
	AidPotTriesPerLevel is the option for "X".
	AidPotTriesReset is the option for "Y".
	AidPotWaitTime is the option for "Z".
--]]
AidPotTriesPerLevel = 3			-- 3 tries, that are reset
AidPotTriesReset = 60000		-- every 60 seconds,
AidPotWaitTime = 200			-- waiting .2 seconds in case of lag

----------------------------------
-- Auto-Homunculus Resurrection --
----------------------------------

--[[ Enable use of automatically casting Homunculus Resurrection

	NOTE: Auto-Homunculus Resurrection only works if the Homunculus is close
		enough to the Alchemist. This script will automatically attempt
		to bring the homunculus within range, but that may not always
		be feasible. I would not recommend relying on this to work every
		time.

	NOTE: If the homunculus actually dies, the script stops. In the event that
		the resurrection is successful, the script will start over as if you
		had just logged in. Please keep this in mind.

--]]
AutoRes = false

--[[
	Begin trying to cast Homunculus Resurrection when the Homunculus's HP goes
	below a certain amount/percent

	NOTE: Auto-Homunculus Resurrection will only be attempted if the homunculus
		is losing life. It stops trying to cast when the homunculus stops
		losing life so that you may gain control of the alchemist once again

	NOTE: If it stops trying to cast because the homunculus is no longer losing life,
		aid-potion may still be attempted if monsters are attacking the homunculus

--]]
AutoResHP = 15

-- Use an "intelligent" routine to decide when a Homunculus Resurrection is necessary
AutoResSmart = true

-- Time (in milliseconds) between uses of Resurrection
AutoResDelay = 400

-----------------------------
-- Amistr-specific Options --
-----------------------------

--[[ If SupportSkills are allowed, Amistr will keep Amistr Bulwark
	active (SP permitting), as long as either it or its owner
	have this many or more things attacking them

	Setting either option to 0 will disable the usage for only
	that actor (owner alchemist or homunculus)
--]]
AmistrBulwarkHomun = 6			-- Amistr's attackers
AmistrBulwarkOwner = 3			-- Alchemist's attackers

-- Minimum level of Amistr Bulwark to use
AmistrBulwarkMinLevel = 1

--[[ If support skills are allowed, Amistr will attempt to use castling
	whenever the owner has this many or more things attacking them
	
	This option will take precedence over Amistr Bulwark
--]]
CastlingAttackers = 5

--[[ But Amistr will only use castling if the owner's HP is equal to
	or lower than a certain amount.
	
	NOTE: Be sure to check PercentHP in Misc. Options below
--]]
CastlingHP = 100

-- The minimum amount of time in milliseconds between casting Castling
CastlingDelay = 500

----------------------------
-- Filir-specific Options --
----------------------------

-- Use duration as casting delay (Filir's Skills are bugged so you can spam them)
FilirUseDuration = true

-- Level of Flitting to use
FlittingLevel = 5

--[[ Use flitting on monsters that (on average) are targetted for at least this
	number of milliseconds

	NOTE: This includes time it takes to move up next to them as well.

	NOTE: To always use flitting as soon as it wears off, set this to 0
--]]
FlittingMinTicks = 0

-- Begin saving SP for flitting (again) this many milliseconds after the last use
FlittingSPSaveTicks = 20000

--------------------------
-- Lif-specific Options --
--------------------------

--[[ Minimum level of Healing Hands to use

	NOTE: Healing Hands will only be used if "HealSkills" (above, in "Other Skill Options")
		is set to true
--]]
MinHealingHands = 1

-- Amount of HP that the alchemist should be at or below before Lif casts Healing Hands
HealingHandsHP = 50

-- Number of things that should be attacking the alchemist before Lif uses Healing Hands
--	(setting to 0 will allow usage outside of combat as well)
HealingHandsMinEnemies = 0

---------------------------------
-- Vanilmirth-specific Options --
---------------------------------

--[[ The only Vanilmirth-specific options deal with Chaotic Blessings.

	If it decides that either itself or the alchemist are below a certain
	amount/percent of HP, it will attempt to use Chaotic Blessings to heal them.

	However, depending on what the max level of Chaotic Blessings is, it will pick
	the most efficient level to use in order to have the highest chance to heal
	for the most amount.

	NOTE: These are only enabled if "HealSkills" (above, in "Other Skill Options")
		is set to true

	NOTE: Set either (or both) of the HP values to 0 to disable
--]]

-- This is the HP that the alchemist should be at before Vanilmirth tries to heal them
ChaoticBlessingsOwnerHP = 50

--[[ This is the HP that the homunculus should be at before Vanilmirth tries to heal itself

	NOTE: There is a bug in RO that makes it so ChaoticBlessings does not heal the
		homunculus when it is casted. Until this bug is fixed, I recommend
		leaving ChaoticBlessingsHomunHP at 0
--]]
ChaoticBlessingsHomunHP = 0

-- This the max level of Chaotic Blessings to use
MaxChaoticBlessings = 5

-- Allow use of chaotic blessings while the homunculus has a target lock?
ChaoticBlessingsWhileLocked = false

-------------------------------
-- Anti-Position Lag Options --
-------------------------------

-- Enable anti-position lag?
AntiPosLag = true

-- How many times should we try for each move command?
PosLagTries = 3

-- How long should it wait in between tries (in milliseconds)
PosLagDelay = 500

-------------------
-- Misc. Options --
-------------------

-- Do not allow automatic usage of Alchemist skills, unless the alchemist
-- will not go below this amount/percent of SP
OwnerMinSP = 0

-- SP numbers are percents (as opposed to absolute amounts)
PercentSP = false

-- HP values are percents rather than amounts
PercentHP = true

-- Wait this number of milliseconds before starting the AI logic
-- (Idea from Fallen~Angel)
AIStartWait = 1000

--[[ Assume PVP zone

	Only enable this if you expect to only go into PVP areas,
	because it may cause subtle unexpected behavior outside
	of PVP areas.
--]]
AssumePVP = false

--[[ Assume WoE zone

	Only enable this if you expect to ONLY go into WoE zones.
--]]
AssumeWoE = false

--[[ Autodetect PVP/WoE zones?

	Enabling this will allow the homunculus to try to attack itself when it
	first enters a map. If it is able to attack itself, then it will know it
	is in a PVP zone.

	NOTE: This only takes effect if AssumePVP and AssumeWoE are both false.

	NOTE: WoE zones can ONLY be auto-detected by Filirs and Vanilmirths. :(

	NOTE: You may want to set MinSP to at least: 4 (filir) / 22 (vani), so that
		the homunculus will always have enough SP to use a skill against
		itself when it enters a WoE zone. If it does not have enough SP
		when it enters, it will not be able to distinguish WoE separately
		from PvM.

	WARNING: This may cause your homunculus to kill itself if it spawns in
		a WoE zone with low hp

	UPDATE: GRAVITY fixed the bug that allowed homunculi to attack themselves,
		but RampageAI now uses other methods to determine if its in a PVP zone.

--]]
CheckPVP = true

--[[ Move-under as an attack command

	In certain circumstances, the Ragnarok Online client will become bugged
	and disallow the alt+right click of any enemy, making it impossible to
	manually instruct the homunculus to attack an enemy. This option
	will allow you to alt+right click on any square within X tiles of an
	enemy to tell the homunculus to attack it. If there are multiple within
	the range, it will select the closest one.
	
	This option is X.
	
	Setting this option to 0 will only allow alt+right clicking underneath
	the monster, and is the recommended setting.
	
	NOTE: This option will be disabled if Advanced Movement Commands are
		enabled. However, There is an AMC to instruct the homunculus
		to attack the target.
--]]
MoveIsAttackWhenDistance = 0
